# -*- coding: UTF-8 -*-
#(c) 2012 by mightbemighty (mightbemighty@gmail.com)
from enigma import eServiceReference, ePicLoad, getDesktop

from Components.AVSwitch import AVSwitch
from Components.config import config
from Components.Label import Label
from Components.Pixmap import Pixmap
from Screens.MessageBox import MessageBox
from Screens.Screen import Screen

from base_media_backend import BaseMediaBackend

import logging
log = logging.getLogger('openair')

class Enigma2MediaBackend(BaseMediaBackend):
	def __init__(self, session, username=None, password=None):
		#host and port are fake, we're native and don't need them!
		BaseMediaBackend.__init__(self, "localhost", 80, username, password)
		self.session = session
		self.__service = None
		self.__moviePlayer = None
		self.__picturePlayer = None
		self.onPlayStateChanged = []

	def __onPlayStateChanged(self, state):
		for fnc in self.onPlayStateChanged:
			fnc(state)

	def _picturePlayerClosed(self):
		self.__picturePlayer = None

	def _moviePlayerClosed(self):
		self.__moviePlayer = None

	def getSeek(self):
		service = self.session.nav.getCurrentService()
		if service is None:
			return None

		seek = service.seek()
		if seek is None or not seek.isCurrentlySeekable():
			return None

		return seek

	def getPosition(self):
		seek = self.getSeek()
		if seek is None:
			return 0
		pos = seek.getPlayPosition()
		if pos[0]:
			return 0
		return pos[1]

	def getLength(self):
		seek = self.getSeek()
		if seek is None:
			return 0
		length = seek.getLength()
		if length[0]:
			return 0
		return length[1]

	def cleanup(self):
		"""
		Called when airplayer is about to shutdown.
		"""
		self.stop_playing()

	def stop_playing(self):
		"""
		Stop playing media.
		"""
		if self.__moviePlayer != None:
			self.__moviePlayer.stop()
			self.__moviePlayer.close()
		if self.__picturePlayer != None:
			self.__picturePlayer.close()

	def play_movie(self, url, pos = 0):
		"""
		Play a movie from the given location.
		"""
		log.debug("Will now play %s", url)
		self.__service = eServiceReference(4097, 0, url)
		if self.__moviePlayer is None:
			self.__moviePlayer = self.session.openWithCallback(self._moviePlayerClosed, AirMoviePlayer, restoreService = True)
			self.__moviePlayer.onPlayBackStateChanged.append(self.__onPlayStateChanged)

		self.set_start_position(pos)
		self.__moviePlayer.playService(self.__service)

	def show_picture(self, data):
		"""
		Show a picture.
		@param data raw picture data.
		"""
		#raise NotImplementedError
		if self.__picturePlayer:
			self.__picturePlayer.showPic(data)
		else:
			self.__picturePlayer = self.session.openWithCallback(self._picturePlayerClosed, AirPicturePlayer, data)

	def notify_started(self):
		"""
		Notify the user that Airplayer has started.
		"""
		pass

	def pause(self):
		"""
		Pause media playback.
		"""
		log.debug("pause")
		if self.__moviePlayer != None:
			return self.__moviePlayer.pause()
		return False

	def play(self):
		"""
		Play media
		"""
		log.debug("play")
		if self.__moviePlayer != None:
			self.__moviePlayer.unpause()
		return False

	def get_player_position(self):
		"""
		Get the current videoplayer positon.
		@returns int current position, int total length
		"""
		log.debug("get_player_position")
		if self.__moviePlayer != None:
			pos = self.getPosition() / 90000
			len = self.getLength() / 90000
			log.debug("pos=%s, len=%s", pos, len)
			return pos, len
		return 0,0

	def is_playing(self):
		"""
		Return wether the backend is currently playing any media.

		@returns boolean
		"""
		log.debug("is_playing")
		if self.__moviePlayer != None:
			return self.__moviePlayer.isPlaying()

		return False

	def set_player_position(self, position):
		"""
		Set the current videoplayer position.

		@param position integer in seconds
		"""
#		raise NotImplementedError
		log.debug("set_player_position")
		if self.__moviePlayer != None:
			self.__moviePlayer.seekTo(position)

	def set_player_position_percentage(self, percentage_position):
		"""
		Set current videoplayer position, in percentage.

		@param percentage_position float
		"""
#		raise NotImplementedError
		log.debug("set_player_position_percentage")
		if self.__moviePlayer != None:
			self.__moviePlayer.seekTo(percentage_position, True)

	def set_start_position(self, percentage_position):
		"""
		Play media from the given location

		@param percentage_position float
		"""
#		raise NotImplementedError
		log.debug("set_start_position")
		if self.__moviePlayer != None:
			self.__moviePlayer.setStartPositionPercentage(percentage_position)

########################## Simple enigma2 MoviePlayer
from enigma import iPlayableService, eTimer

from Screens.Screen import Screen
from Screens.ChoiceBox import ChoiceBox
from Screens.InfoBarGenerics import InfoBarNotifications, InfoBarSeek, InfoBarShowHide, InfoBarAudioSelection, InfoBarCueSheetSupport, InfoBarSubtitleSupport

from Components.ActionMap import ActionMap
from Components.ServiceEventTracker import ServiceEventTracker

class AirMoviePlayer(Screen, InfoBarNotifications, InfoBarSeek, InfoBarShowHide, InfoBarAudioSelection, InfoBarCueSheetSupport, InfoBarSubtitleSupport):
	ENABLE_RESUME_SUPPORT = True
	ALLOW_SUSPEND = True

	STATE_STOPPED = 0
	STATE_PLAYING = 1
	STATE_PAUSED = 2

	def __init__(self, session, service = None, start_position = 0, restoreService = True, infoCallback = None, getNextService = None, getPrevService = None, stopCallback = None):
		Screen.__init__(self, session)
		self.skinName = "MoviePlayer" # we force the infobar-skin of the standard movieplayer

		InfoBarNotifications.__init__(self)
		InfoBarSeek.__init__(self)
		InfoBarShowHide.__init__(self)
		InfoBarAudioSelection.__init__(self)
		InfoBarSubtitleSupport.__init__(self)
		InfoBarCueSheetSupport.__init__(self)
		#TODO FIX THIS HACK
		# currently we just want to be able to resume playback (if supported by e2),
		# for now we don't care about cutting or jumpmarks or anything like that...
		del self["CueSheetActions"]

		self.session = session
		self.service = service
		self.infoCallback = infoCallback
		self.getNextServiceCB = getNextService
		self.getPrevServiceCB = getPrevService
		self.stopCB = stopCallback
		self.callback = None
		self.screen_timeout = 5000
		self.nextservice = None

		self.is_closing = False
		self.restoreService = restoreService
		self.oldService = None
		self._playState = self.STATE_STOPPED
		self._isStartPositionSet = False

		self.returning = False
		self.onPlayBackStateChanged = []
		self._startPositionPercentage = start_position
		self.__forcePlayEventTimer = eTimer()
		self.__forcePlayEventTimer.callback.append(self.__forcePlayEvent)

		self["actions"] = ActionMap(["OkCancelActions", "InfobarSeekActions", "MediaPlayerActions", "MovieSelectionActions"],
		{
				"cancel": self.leavePlayer,
				"stop": self.leavePlayer,
				"playpauseService": self.playpauseService,
				"previous":  self.playPrev,
				"next": self.playNext,
				"showEventInfo": self.showVideoInfo,
			}, prio=0)

		self.__event_tracker = ServiceEventTracker(screen=self, eventmap=
		{
			iPlayableService.evUpdatedInfo: self._evUpdatedInfo,
		})

		self.onClose.append(self.__onClose)

	"""
	evStart is issued very soon, so we usually won't have duration & position available at that point.
	We react on all evUpdatedInfo until we were actually able to seek to the desired startposition
	If whatever we play is not seekable at all, self._isStartPositionSet will be set to true by self.seekTo
	"""
	def _evUpdatedInfo(self, *args):
		if self._startPositionPercentage > 0:
			if not self._isStartPositionSet:
				log.info("[AirMoviePlayer] _evUpdatedInfo args=%s")
				self.seekTo(self._startPositionPercentage, True)
		else:
			if not self._isStartPositionSet:
				seekable = self.getSeek()
				len = seekable.getLength()
				if not len[0]:
					length = float(len[1]) / 90000
					if length > 0:
						pos = seekable.getPlayPosition()
						if not pos[0]:
							log.debug("[AirMoviePlayer] _evUpdatedInfo, startPosition is 0, pos & len avail, issue evPlay")
							self._isStartPositionSet = True
							self._evPlay()

	def _evPlay(self, *args):
		log.debug("[AirMoviePlayer] _evStart, self._startPositionPercentage=%s", self._startPositionPercentage)
		self.__forcePlayEventTimer.stop()
		if self._isStartPositionSet:
			self.__onPlayStateChanged(self.STATE_PLAYING)

	def _evStop(self, *args):
		log.debug("[AirMoviePlayer] _evStop")
		self.__onPlayStateChanged(self.STATE_STOPPED)

	def _evError(self, *args):
		log.debug("[AirMoviePlayer] _evError %s", str(args))
		self.__onPlayStateChanged(self.STATE_STOPPED)

	def __onPlayStateChanged(self, state):
		self._playState = state
		for fnc in self.onPlayBackStateChanged:
			fnc(self._playState)

	def __onClose(self):
		if self.stopCB != None:
				self.stopCB()
		self.stop()

	def __forcePlayEvent(self):
		log.debug("[AirMoviePlayer] __forcePlayEvent")
		self._isStartPositionSet = True
		self._evPlay()

	def isPlaying(self):
		return self._playState == self.STATE_PLAYING or self._playState == self.STATE_PAUSED

	def createSummary(self):
		return SimpleLCDScreen

	def playNext(self):
		if self.getNextServiceCB != None:
			newservice = self.getNextServiceCB()
			log.debug("[AirMoviePlayer] playNext newservice is %s", newservice)
			if newservice:
				self.playService(newservice)
				return

		self.leavePlayerConfirmed([True, "quit"])

	def playPrev(self):
		if self.getPrevServiceCB != None:
			newservice = self.getPrevServiceCB()
			log.debug("[AirMoviePlayer] playPrev newservice is %s", newservice)
			if newservice:
				self.playService(newservice)

		self.leavePlayerConfirmed([True, "quit"])

	def playAgain(self):
		self.play()

	def setStartPositionPercentage(self, pos):
		self._startPositionPercentage = pos

	def playService(self, newservice):
		log.info("[AirMoviePlayer] playService %s", newservice.toString())
		self.stop(False) #stop playing but don't tell the client (it confuses some of them)
		self.service = newservice
		self.play()

	def play(self):
		if self._playState == self.STATE_STOPPED:
			self.oldService = self.session.nav.getCurrentlyPlayingServiceReference()

		text = self.service.getPath()
		text = text.split('/')[-1]
		log.info("[AirMoviePlayer] play - text=%s", text)
		self.summaries.setText(text, 3)

		self._isStartPositionSet = False
		self.session.nav.playService(self.service)
		self.session.nav.getCurrentService().streamed().setBufferSize(long(config.plugins.openair.buffersize.value) * 1024 * 1024)

		self.setSeekState(self.SEEK_STATE_PLAY)
		self.__forcePlayEventTimer.start(2500, True)

	def seekTo(self, target, percentage = False):
		log.info("[AirMoviePlayer] seekTo: target=%s, percentage=%s", target, percentage)
		if target >= 0:
			length = 0
			pos = 0
			service = self.session.nav.getCurrentService()
			if self.isSeekable():
				log.debug("[AirMoviePlayer] we can seek")
				seekable = self.getSeek()
				tmp = seekable.getLength()
				if not tmp[0]:
					log.debug("[AirMoviePlayer] getLength is OK")
					length = float(tmp[1]) / 90000

					tmp = seekable.getPlayPosition()
					if not tmp[0]:
						pos = float(tmp[1]) / 90000

						if percentage:
							target = length * target / 100

						log.info("[AirMoviePlayer] seeking to %s seconds", target)
						to = long(target * 90000) #PTS everywhere!

						self.session.nav.getCurrentService().streamed().setBufferSize(long(config.plugins.openair.buffersize.value) * 1024 * 1024)
						self.doSeek(to)
						if not self._isStartPositionSet:
							self._isStartPositionSet = True
							self._evPlay()
						return True
			else:
				if not self._isStartPositionSet:
					self._isStartPositionSet = True
					self._evPlay()
				log.warning("[AirMoviePlayer] seekTo: not seekable!")
		return False

	def pause(self):
		if self._playState == self.STATE_PLAYING:
			res = self.setSeekState(self.SEEK_STATE_PAUSE)
			if res:
				self.__onPlayStateChanged(self.STATE_PAUSED)
			return res
		return False

	def unpause(self):
		if self._playState == self.STATE_PAUSED:
			res = self.setSeekState(self.SEEK_STATE_PLAY)
			if res:
				self.__onPlayStateChanged(self.STATE_PLAYING)
			return res
		return False

	def stop(self, sendEvent=True):
		if self._playState != self.STATE_STOPPED:
			if sendEvent:
				self._evStop()
			else:
				self._playState = self.STATE_STOPPED

			if self.restoreService and self.oldService:
				log.debug("[AirMoviePlayer] restoring previous service (%s)", self.oldService.toString())
				self.session.nav.playService(self.oldService)
			else:
				log.debug("[AirMoviePlayer] NOT restoring previous service! restoreService=%s, oldService=%s", self.restoreService, self.oldService)
				self.session.nav.stopService()

	def showVideoInfo(self):
		if self.infoCallback != None:
			self.infoCallback(self.service)

	def handleLeave(self, ask = True, error = False):
		self.is_closing = True
		if ask:
			list = [(_("Yes"), "quit"),
					(_("No, but play video again"), "playAgain")]

			if self.getNextServiceCB != None:
				list.append( (_("Yes, but play next service"), "playnext") )
			if self.getPrevServiceCB != None:
				list.append( (_("Yes, but play previous service"), "playprev") )

			if error is False:
				self.session.openWithCallback(self.leavePlayerConfirmed, ChoiceBox, title=_("Stop playing this movie?"), list = list)
			else:
				self.session.openWithCallback(self.leavePlayerConfirmed, ChoiceBox, title=_("No playable video found! Stop playing this movie?"), list = list)
		else:
			self.leavePlayerConfirmed([True, "quit"])

	def leavePlayer(self):
		self.handleLeave()

	def leavePlayerConfirmed(self, answer):
		answer = answer and answer[1]
		if answer == "quit":
			self.close()

		elif answer == "playnext":
			self.playNext()

		elif answer == "playprev":
			self.playPrev()

		elif answer == "playAgain":
			self.playAgain()

	def doEofInternal(self, playing):
		self.close()

class AirPicturePlayer(Screen):
	TMP_FILE = "/tmp/.tmpPicture.jpg"
	width = getDesktop(0).size().width()
	height = getDesktop(0).size().height()

	skin="""<screen position="0,0" size="%(width)s,%(height)s" flags="wfNoBorder">
		<eLabel position="0,0" size="%(width)s,%(height)s" zPosition="0" backgroundColor="#00000000" />
		<widget name="picture" position="0,0" size="%(width)s, %(height)s" zPosition="1" alphatest="on"/>
	</screen>
	""" % { 'height' : height, 'width' : width}

	def __init__(self, session, data=None):
		Screen.__init__(self, session)

		self._data = data

		self._picture = Pixmap()
		self["picture"] = self._picture

		self._picload = ePicLoad()
		self._picload.PictureData.get().append(self._finishDecode)
		self._pixmap = None

		self["actions"] = ActionMap(["OkCancelActions"],
		{
			"cancel" : self.close
		}, -1)

		self.onLayoutFinish.append(self._onLayoutFinish)
		self.onClose.append(self._onClose)

	def _onLayoutFinish(self):
		scale = AVSwitch().getFramebufferScale()
		self._picload.setPara([self.width, self.height, scale[0], scale[1], 0, 1, "#00000000"])
		if self._data != None:
			self.showPic(self._data)

	def _onClose(self):
		del self._picload
		del self._pixmap

	def showPic(self, data):
		self._data = data
		try:
			with open(self.TMP_FILE, "w") as f:
				f.write(self._data)
			self._picload.startDecode(self.TMP_FILE)
		except IOError:
			msgbox = self.session.open(MessageBox, _("Error writing picture data!"), type = MessageBox.TYPE_ERROR, timeout = 5)
			msgbox.setTitle(_("Failed to write data"))

	def _finishDecode(self, info = ""):
		self._pixmap = self._picload.getData()
		if self._pixmap:
			self._picture.instance.setPixmap(self._pixmap.__deref__())
		else:
			msgbox = self.session.open(MessageBox, _("Error decoding picture!"), type = MessageBox.TYPE_ERROR, timeout = 5)
			msgbox.setTitle(_("Failed to decode!"))

class SimpleLCDScreen(Screen):
	skin = (
	"""<screen name="MediaPlayerLCDScreen" position="0,0" size="132,64" id="1">
		<widget name="text1" position="4,0" size="132,18" font="Regular;16"/>
		<widget name="text2" position="4,19" size="132,14" font="Regular;10"/>
		<widget name="text3" position="4,34" size="132,14" font="Regular;10"/>
		<widget name="text4" position="4,49" size="132,14" font="Regular;10"/>
	</screen>""",
	"""<screen name="MediaPlayerLCDScreen" position="0,0" size="96,64" id="2">
		<widget name="text1" position="0,0" size="96,18" font="Regular;16"/>
		<widget name="text2" position="0,19" size="96,14" font="Regular;10"/>
		<widget name="text3" position="0,34" size="96,14" font="Regular;10"/>
		<widget name="text4" position="0,49" size="96,14" font="Regular;10"/>
	</screen>""")

	def __init__(self, session, parent):
		Screen.__init__(self, session)
		self["text1"] = Label("Playing")
		self["text2"] = Label("")
		self["text3"] = Label("")
		self["text4"] = Label("")

	def setText(self, text, line):
		if line == 1:
			if len(text) > 15:
				#TODO remove this hack
				if text[-4:-3] == ".":
					text = text[:-4]
				if len(text) > 15:
					text = text[-15:]
		else:
			if len(text) > 20:
				#TODO remove this hack
				if text[-4:-3] == ".":
					text = text[:-4]
				if len(text) > 20:
					text = text[-20:]
		textleer = "	"
		text = text + textleer*10
		if line == 1:
			self["text1"].setText(text)
		elif line == 2:
			self["text2"].setText(text)
		elif line == 3:
			self["text3"].setText(text)
		elif line == 4:
			self["text4"].setText(text)
