#Copyright (c) 2012, mightbemighty (mightbemighty@gmail.com)

from Components.ActionMap import ActionMap
from Components.config import config, ConfigEnableDisable, ConfigSelection, ConfigSubsection, getConfigListEntry
from Components.ConfigList import ConfigListScreen
from Components.Sources.StaticText import StaticText
from Plugins.Extensions.Bonjour.Bonjour import bonjour
from Plugins.Plugin import PluginDescriptor
from Screens.Screen import Screen

from protocol_handler_twisted import AirplayProtocolHandler
from mediabackends.enigma2_media_backend import Enigma2MediaBackend

from Components.Network import iNetwork

import logging
import appletv

from Tools.RedirectOutput import EnigmaOutput
import sys
class FixedEnigmaOutput(EnigmaOutput):
	def isatty(self):
		return False
	def flush(self):
		pass
sys.stdout = sys.stderr = FixedEnigmaOutput()

config.plugins.openair = ConfigSubsection()
config.plugins.openair.enabled = ConfigEnableDisable(default=True)
config.plugins.openair.buffersize = ConfigSelection([("1", "1 MB"), ("2", "2 MB"), ("4", "4 MB"), ("8", "8 MB"), ("16", "16 MB")])
config.plugins.openair.logtofile = ConfigEnableDisable(default=False)


global instance, session
instance = session = None

class OpenAirConfig(ConfigListScreen, Screen):
	skin = """
		<screen name="OpenAirConfig" position="center,center" size="560,400" title="OpenAir Configuration">
			<ePixmap pixmap="skin_default/buttons/red.png" position="0,0" size="140,40" alphatest="on" />
			<ePixmap pixmap="skin_default/buttons/green.png" position="140,0" size="140,40" alphatest="on" />
			<widget source="key_red" render="Label" position="0,0" zPosition="1" size="140,40" font="Regular;20" halign="center" valign="center" backgroundColor="#9f1313" transparent="1" />
			<widget source="key_green" render="Label" position="140,0" zPosition="1" size="140,40" font="Regular;20" halign="center" valign="center" backgroundColor="#1f771f" transparent="1" />
			<widget name="config" position="5,50" size="550,360" scrollbarMode="showOnDemand" zPosition="1"/>
		</screen>"""

	def __init__(self, session, args=0):
		Screen.__init__(self, session)
		ConfigListScreen.__init__(self, [])

		self["key_red"] = StaticText(_("Cancel"))
		self["key_green"] = StaticText(_("OK"))
		self["setupActions"] = ActionMap(["SetupActions", "ColorActions"],
		{
			"red": self.keyCancel,
			"green": self.keySave,
			"save": self.keySave,
			"cancel": self.keyCancel,
		}, -2)

		self._createSetup()
		self.onLayoutFinish.append(self._layoutFinished)
		config.plugins.openair.enabled.addNotifier(self._enabledChanged, initial_call = False)
		self.onClose.append(self._onClose)

	def _onClose(self):
		global instance
		if instance:
			instance.configure_logging()
		config.plugins.openair.enabled.removeNotifier(self._enabledChanged)

	def _layoutFinished(self):
		self.setTitle(_("OpenAir Configuration"))

	def _enabledChanged(self, enabled):
		self._createSetup()

	def _createSetup(self):
		list = [getConfigListEntry(_("Start OpenAir on boot"), config.plugins.openair.enabled)]
		if config.plugins.openair.enabled.value:
			list.append(getConfigListEntry(_("Buffer Size"), config.plugins.openair.buffersize))
			list.append(getConfigListEntry(_("Debug to logifle"), config.plugins.openair.logtofile))

		self["config"].setList(list)

class OpenAir(object):
	LOGFILE = '/tmp/openair.log'
	def __init__(self, session, port=7000):
		self._session = session
		self._port = port

		self.setDeviceInfoMac()
		self._bonjourService = bonjour.buildService("airplay", self._port, text=bonjour.txtFromDict(appletv.DEVICE_INFO))

	def setDeviceInfoMac(self):
		nics = iNetwork.getConfiguredAdapters()
		if len(nics) > 0:
			nic = nics[0]
			appletv.DEVICE_INFO['deviceid'] = iNetwork.getAdapterAttribute(nic, "mac")

	def run(self):
#		self._session = session
		self.configure_logging()
		self.log.info('Starting OpenAir')

		self._register_bonjour()
		self._register_media_backend()

		self._media_backend.notify_started()
		self._start_protocol_handler()

	def configure_logging(self):
		self.log = logging.getLogger('openair')
		fmt = r"%(asctime)s [%(levelname)s] %(message)s"
		datefmt = r"%Y-%m-%d %H:%M:%S"

		if config.plugins.openair.logtofile.value:
			handler = logging.FileHandler(OpenAir.LOGFILE)
			self.log.setLevel(logging.DEBUG)
		else:
			handler = logging.StreamHandler()
			self.log.setLevel(logging.INFO)

		handler.setFormatter(logging.Formatter(fmt, datefmt))
		self.log.addHandler(handler)

	def _register_bonjour(self):
		bonjour.registerService(self._bonjourService, True)
		print "[OpenAir._register_bonjour] Service '%s' on port '%i' has been registered!" %(self._bonjourService, self._port)

	def _register_media_backend(self):
		self._media_backend = Enigma2MediaBackend(self._session)

	def _start_protocol_handler(self):
		self._protocol_handler = AirplayProtocolHandler(self._port, self._media_backend)
		self._protocol_handler.start()

	def shutdown(self):
		self._protocol_handler.stop()
		self._media_backend.stop_playing()
		bonjour.unregisterService("airplay")

def session(reason, **kwargs):
	global session
	session = kwargs.get('session', None)

def start(reason, **kwargs):
	global instance, session
	if session is not None and reason and config.plugins.openair.enabled.value:
		instance = OpenAir(session)
		instance.run()
	else:
		if instance is not None:
			instance.shutdown()

def setup(session, **kwargs):
	session.open(OpenAirConfig)

def Plugins(**kwargs):
	return [
		PluginDescriptor(where=[PluginDescriptor.WHERE_SESSIONSTART], fnc=session),
		PluginDescriptor(where=[PluginDescriptor.WHERE_NETWORKCONFIG_READ], fnc=start),
		PluginDescriptor(name = "OpenAir", description = "An implementation of apples AirPlay Protocol", where = PluginDescriptor.WHERE_PLUGINMENU, fnc=setup)
	]
